/*
 * SuxPanel version 0.1
 * Copyright (c) 2003 Leandro Pereira <leandro@linuxmag.com.br>
 *
 * This program may be distributed under the terms of GNU General
 * Public License version 2. You should have received a copy of the
 * license with this program; if not, please consult http://www.fsf.org/.
 *
 * This program comes with no warranty. Use at your own risk.
 *
 */

#include "suxpanel.h"
#include <sys/stat.h>
#include <dirent.h>

#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

static GtkWidget   *box;
static GtkTooltips *appbar_tips;

static GtkWidget *btn_from_appdir(const gchar *path, const gchar *dir_name);

static void build_appbar (gchar *dir_name)
{
	DIR *dir;
	GtkWidget *btn;
	struct dirent *sd;
	struct stat st;

	dir = opendir(dir_name);
	if (!dir) return;
	
	if (!box) {
		box = gtk_hbox_new(FALSE, 4);
		gtk_widget_show(box);
	}

	while ((sd = readdir(dir))) {
		gchar *buf;
		
		if (sd->d_name[0] == '.') continue;
		
		buf = g_strdup_printf("%s/%s/AppIcon.xpm", dir_name, sd->d_name);
		
  		if (!stat(buf, &st)) {
			btn = btn_from_appdir(dir_name, sd->d_name);
			gtk_tooltips_set_tip(appbar_tips, btn, sd->d_name, NULL);
			gtk_box_pack_start(GTK_BOX(box), btn, FALSE, FALSE, 0);
		} 

		g_free(buf);
	}
	
	closedir(dir);
}

static gboolean spawn_app(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	gchar *app, *path;
	pid_t child;

	if (event->button != 1) return FALSE;
	
	path = g_object_get_data(G_OBJECT(widget), "path");
	app = g_strdup_printf("%s/%s/AppRun", path, (gchar*)data);
	
	child = fork();
	if (child == -1) {
		GtkWidget *dialog;
		
		dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
			GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
			"Yuck. Can't fork.");
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	} else if (child == 0) {
		char *argv[2];
		
		argv[0] = app;
		argv[1] = 0;
		
		execv(app, argv);
		_exit(127);
	}
	
	g_free(app);
	
	return FALSE;
}

static gboolean enter_notify(GtkWidget *wid, gpointer data)
{
	gtk_widget_modify_bg(GTK_WIDGET(wid), GTK_STATE_NORMAL, 
		&(wid->style->bg[GTK_STATE_SELECTED]));
	
	return FALSE;
}

static gboolean leave_notify(GtkWidget *wid, gpointer data)
{
	gtk_widget_modify_bg(GTK_WIDGET(wid), GTK_STATE_NORMAL, 
		&(wid->parent->style->bg[GTK_STATE_NORMAL]));
	
	return FALSE;
}

static GtkWidget *btn_from_appdir(const gchar *path, const gchar *dir_name)
{
	GtkWidget *btn;
	GtkWidget *pix;
	gchar *buf;
	
	btn = gtk_event_box_new();
	gtk_widget_show(btn);
	
	g_object_set_data(G_OBJECT(btn), "path", g_strdup(path));
		
	buf = g_strdup_printf("%s/%s/AppIcon.xpm", path, dir_name);

	if (g_file_test(buf, G_FILE_TEST_EXISTS)) {		
		pix = gtk_image_new_from_file_scaled(buf, 24, 24);
		gtk_widget_show(pix);
		gtk_container_add(GTK_CONTAINER(btn), pix);
	}

	g_free(buf);

	g_signal_connect(G_OBJECT(btn), "button_press_event", (GCallback)spawn_app, 
				g_strdup(dir_name));
	g_signal_connect(G_OBJECT(btn), "enter_notify_event", (GCallback)enter_notify, 
				NULL);
	g_signal_connect(G_OBJECT(btn), "leave_notify_event", (GCallback)leave_notify, 
				NULL);
				
	gtk_widget_set_size_request(GTK_WIDGET(btn), 24, 28);
	return btn;
}

const gchar *sux_name (void)
{
	return "Application Bar";
}

static void
child_died (int signum)
{
        pid_t child;
        int status; 

        /* Find out which children exited and allow them to die */
        while (1) {
                child = waitpid (-1, &status, WNOHANG);

                if (child == 0 || child == -1)
                        break;
        }

}

void sux_init (SuxModule *sm)
{
	gchar *buf;
	struct sigaction act;
	
	buf = g_strdup_printf("%s/.suxpanel/appbar", g_get_home_dir());
	if (g_file_test(buf, G_FILE_TEST_EXISTS)) {
		act.sa_handler = child_died;
		sigemptyset(&act.sa_mask);
		act.sa_flags = SA_NOCLDSTOP;
		sigaction(SIGCHLD, &act, NULL);

		appbar_tips = gtk_tooltips_new();

		build_appbar(buf);
	
		sm->widget = box;
	} else {
		g_print("*** Did you ran suxpanel-install.sh?\n");
	}
	
	g_free(buf);
}

void sux_fini (SuxModule *sm)
{
	gtk_widget_destroy(sm->widget);
}

void sux_about (void)
{
        GtkAbout *about;
        const gchar *msg[] = {
                ">Written by:",
                        "Leandro Pereira (leandro@linuxmag.com.br)",
                ">Disclaimer:",
                        "This is free software; you can modify and/or distribute it",
                        "under the terms of GNU GPL version 2. See http://www.fsf.org/",
                        "for more information.",
                NULL
        };
        
        about = gtk_about_new("Application Bar", "0.2.2",
                        "Yay to the bar, baby!",
                        msg, IMG_PREFIX "apps.png");
}
